//
//  2014 David Vines
//
// Creative Commons License
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
//
Character = {};

Character.allstats = [ "str", "dex", "agi", "con", "app", "int", "wit", "will" ];
Character.longstats = [ "Strength", "Dexterity", "Agility", "Constitution",
        "Appearance", "Intellect", "Wit", "Will" ];

Character.returnFalse = function(f) {
    return function() {
        try {
            f();
        } catch (e) {
            if (e.stack) {
                alert(e + "\n\nStack Trace\n-----------------\n" + e.stack);
            } else {
                alert(e);
            }
        }
        return false;
    };
};

Character.incrementVersion = function() {
    character.characterVersion += 1;
    character.lastEditDate = (new Date(Date.now())).toLocaleString();
    Character.populateJSON();
};

Character.updateMonths = function() {
    character.birthday = character.genre.dayMonthYearAsText(character.birthtime, character.birthmonth);
    character.gameday = character.genre.dayMonthYearAsText(character.gametime,character.gamemonth);
};

Character.templates = {
    interest : {
        skill : "Numeracy",
        ip : 1,
        teacher : false
    }
};

Character.setIdText = function(id, text, tooltip) {
    var element = document.getElementById(id);
    var newText = document.createTextNode(text);
    if (tooltip) {
        element.title = tooltip;
    }
    if (element.hasChildNodes()) {
        element.replaceChild(newText, element.firstChild);
    } else {
        element.appendChild(newText);
    }
};

Character.setTextArea = function(field, text) {
    var element = document.getElementById(field);
    element.value = text;
    element.defaultValue = text;
};

Character.setOption = function(options, choice) {
    var found = false;
    for (var j = 0; j < options.length; j++) {
        if (choice.toString() === options[j].text) {
            options.selectedIndex = j;
            found = true;
        }
    }
    if (!found) {
        var newOption = new Option(choice.toString(), choice.toString(), true,
                true);
        options.add(newOption);
    }
};

Character.setChecked = function(button, choice) {
    button.checked = choice;
    button.defaultChecked = choice;
};

Character.deleteElement = function(idOrElement) {
    var element = ((typeof idOrElement === "string") ? document.getElementById(idOrElement) : idOrElement);
    if (element) {
        element.parentNode.removeChild(element);
    }
};

Character.durationLength = function(dur) {
    if (dur === 0)
        return "0 months";
    if (dur === 1)
        return "1 month";
    if (dur < 12)
        return dur + " months";
    var years = Math.floor(dur / 12);
    dur = dur % 12;

    if (years === 1) {
        if (dur === 0)
            return "1 year";
        return "1 year, " + Character.durationLength(dur);
    } else {
        if (dur === 0)
            return years + " years";
        return years + " years, " + Character.durationLength(dur);
    }
};

Character.getFluencyPoints = function(p) {
    var endFluency = (character.periods[p].endmonth < 179) ? character.periods[p].endmonth
            : 179;
    var fluencyMonths = endFluency - character.periods[p].startmonth + 1;
    if (fluencyMonths < 0) {
        return 0;
    }
    var points = (character.periods[p].startmonth == 60) ? 1000 : 0;
    points += fluencyMonths * 8;
    for (var anniversary = 72; anniversary <= 180; anniversary += 12) {
        if ((character.periods[p].startmonth <= anniversary) && (character.periods[p].endmonth >= anniversary)) {
            points += 4;
        }
    }
    return points;
};

Character.updateFluencyPoints = function(p) {
    var points = Character.getFluencyPoints(p);
    if (points > 0) {
        Character.setIdText("period"+p+"-fluencypoints",points+" points");
    }
};

Character.deleteAllChildren = function(node) {
    while (node.firstChild) {
        node.removeChild(node.firstChild);
    }
};

Character.addInterestRow = function(p,table,index) {
    var template = document.getElementById("period@@-interest0");
    var row = template.cloneNode(true);
    var next = index;
    Character.updateNode(row,p);
    Character.updateForm(row,next);
    row.id = "period"+p+"-interest"+next;
    table.appendChild(row);
    Character.addSkillRow(p,next);
    Character.populateJSON();
};

Character.addInterest = function(skillname) {
    return Character.returnFalse(function() {
        // Work out which tab is active
        var tab;
        var tabbersheet = document.getElementById("sheet");
        if (tabbersheet.tabber) {
            tab = tabbersheet.tabber.getActiveTab();
        }
        if (tab && (tab.id.slice(0,6) == "period")) {
            var p = Number(tab.id.slice(6));
            var table = document.getElementById("period"+p+"-details").firstChild.nextSibling;
            if (character.periods[p].iptotal < character.interestPoints) {
                Character.addInterestKnowingPeriod(p,table,skillname);
                Character.incrementVersion();
                Character.updateStatus(p);
            }
        }
    });
};

Character.updateTabs = function() {
    var tabbersheet = document.getElementById("sheet");
    if (tabbersheet.tabber) {
        tabbersheet.tabber.reinit(tabbersheet);
    }
};

Character.updateArmour = function(armour) {
    return Character.returnFalse(function() {
        character.armour = armour;
        Character.incrementVersion();
        Character.populateJSON();
    });
};

Character.updateShield = function(shield) {
    return Character.returnFalse(function() {
        character.shield = shield;
        if (character.shield.locations === 0) {
            character.shieldLocations = [];
        } else if (character.shield.locations === 1) {
            character.shieldLocations = [ "shieldArm", "chest"];
        } else if (character.shield.locations === 2) {
            character.shieldLocations = [ "shieldArm", "chest","head"];
        } else if (character.shield.locations === 3) {
            character.shieldLocations = [ "shieldArm", "chest","head","abdomen"];
        }
        Character.populateShieldLocations();
        Character.incrementVersion();
        Character.populateJSON();
    });
};

Character.updateShieldLocations = function() {
    return Character.returnFalse(function() {
        var locs;
        locs = ["head", "chest", "abdomen"];
        character.shieldLocations = ["shieldArm"];
        for(var i=0; i<locs.length; i++) {
            if (document.getElementById("coverage"+locs[i]).checked) {
                character.shieldLocations.push(locs[i]);
            }
        }
        Character.incrementVersion();
        Character.populateJSON();
    });
};

Character.populateShieldLocations = function() {
    var containsShieldLocation = function(location) {
        return function(x) { return x === location; };
    };
    document.getElementById("trcoverageinfo").firstChild.data = character.shield.text;
    var head =  document.getElementById("coveragehead");
    var chest = document.getElementById("coveragechest");
    var abdomen = document.getElementById("coverageabdomen");
    if (character.shield.locations === 0) {
        head.type = "checkbox";
        chest.type = "checkbox";
        abdomen.type = "checkbox";
        head.checked = false;
        chest.checked = false;
        abdomen.checked = false;
        head.disabled = true;
        chest.disabled = true;
        abdomen.disabled = true;
    } else if (character.shield.locations === 1) {
        head.type = "radio";
        chest.type = "radio";
        abdomen.type = "radio";
        head.checked = character.shieldLocations.some(containsShieldLocation("head"));
        chest.checked = character.shieldLocations.some(containsShieldLocation("chest"));
        abdomen.checked = character.shieldLocations.some(containsShieldLocation("abdomen"));
        head.disabled = false;
        chest.disabled = false;
        abdomen.disabled = false;
        head.onchange = Character.updateShieldLocations();
        chest.onchange = Character.updateShieldLocations();
        head.onchange = Character.updateShieldLocations();
    } else if (character.shield.locations === 2) {
        head.type = "radio";
        chest.type = "checkbox";
        abdomen.type = "radio";
        head.checked = character.shieldLocations.some(containsShieldLocation("head"));
        chest.checked = true;
        abdomen.checked = character.shieldLocations.some(containsShieldLocation("abdomen"));
        head.disabled = false;
        chest.disabled = true;
        abdomen.disabled = false;
        head.onchange = Character.updateShieldLocations();
        abdomen.onchange = Character.updateShieldLocations();
    } else if (character.shield.locations === 3) {
        head.type = "checkbox";
        chest.type = "checkbox";
        abdomen.type = "checkbox";
        head.checked = true;
        chest.checked = true;
        abdomen.checked = true;
        head.disabled = true;
        chest.disabled = true;
        abdomen.disabled = true;
    }
};

Character.updateFavouredWeapons = function() {
    return Character.returnFalse(function() {
        character.favouredWeapons = [];
        for(var i=0; i<Rules.weapons.length; i++) {
            var checkbox = document.getElementById("weapon-"+Rules.weapons[i].name);
            if (checkbox && checkbox.checked) {
                character.favouredWeapons.push(Rules.weapons[i]);
            }
        }
        Character.incrementVersion();
        Character.populateJSON();
    });
};

Character.populateEquipment = function() {
    var containsWeapon = function(weapon) { return function(x) { return x.name === weapon.name; }; };
    var containsGenre = function(x) { return x === character.genre.toString(); };
    var radioequip = function(radioname,array,setting,update) {
        var b = 0;
        var button,span;
        for(var i=0; i<array.length; i++) {
            if (array[i].genre.some(containsGenre)) {
                span = document.getElementById("td"+radioname+b);
                button = document.getElementById(radioname+b);
                span.className = "";
                button.nextSibling.data = array[i].name;
                button.checked = (array[i].name === setting.name);
                button.onchange = update(array[i]);
                b++;
            }
        }
        // Clear out the other buttons, if any
        for(;;b++) {
            span = document.getElementById("td"+radioname+b);
            if (!span) break;
            button = document.getElementById(radioname+b);
            span.className = "hidden";
            if (button.checked) {
                button.checked = false;
                document.getElementById(radioname+"0").checked = true;
            }
        }

    };
    
    radioequip("armour",Rules.armour,character.armour,Character.updateArmour);
    radioequip("shield",Rules.shields,character.shield,Character.updateShield);
    
    // shield locations
    Character.populateShieldLocations();
    
    // Weapons are a set of checkboxs, (6 columns)
    var columns = 6;
    var weapons = document.getElementById("weapons");
    var newweapons = document.createDocumentFragment();
    var newrow = document.createElement("tr");
    newrow.id = "weapons";
    var column = 0;
    for(var i=0; i<Rules.weapons.length; i++) {
        if (Rules.weapons[i].genre.some(containsGenre)) {
            var newcheckbox = document.createElement("input");
            newcheckbox.type = "checkbox";
            newcheckbox.id = "weapon-"+Rules.weapons[i].name;
            newcheckbox.checked = character.favouredWeapons.some(containsWeapon(Rules.weapons[i]));
            newcheckbox.onchange = Character.updateFavouredWeapons();
            var newcell = document.createElement("td");
            newcell.appendChild(newcheckbox);
            newcell.appendChild(document.createTextNode(Rules.weapons[i].name));
            newrow.appendChild(newcell);
            column++;
            if (column == columns) {
                newweapons.appendChild(newrow);
                newrow = document.createElement("tr");
                column = 0;
            }
        }
    }
    if (column > 0) {
        newweapons.appendChild(newrow);
    }
    while(weapons.nextSibling) {
        weapons.parentNode.removeChild(weapons.nextSibling);
    }
    weapons.parentNode.replaceChild(newweapons,weapons);
};

Character.populatePeriodMonths = function(p) {
    Character.setIdText("period"+p+"-enddatelength"," A period of "+Character.durationLength(character.periods[p].endmonth - character.periods[p].startmonth  + 1));
    if (character.periods[p].startmonth === character.periods[p].endmonth) {
        Character.setIdText("period"+p+"-daterange","The month of "+character.periods[p].startmonthtext);
    } else if (character.periods[p].startmonth < character.periods[p].endmonth) {
        Character.setIdText("period"+p+"-daterange",character.periods[p].startmonthtext+" to "+character.periods[p].endmonthtext);
    } else {
        Character.setIdText("period"+p+"-daterange","Future interests");
    }
};

Character.populateSkills = function() {
    var i,j;
    var endingMonthText = character.genre.dateMonthAsText(character.periods[character.periods.length-1].endmonth+character.birthmonth);
    
    var points = {};
    var addPoints = function(points,name,dp) {
        if (points[name]) {
            points[name] += dp;
        } else {
            points[name] = dp;
        }
    };
    for(i=0; i<character.periods.length; i++) {
        // Childhood fluency
        if (Character.getFluencyPoints(i) > 0) {
            addPoints(points,"Fluency("+character.periods[i].childhoodLanguage+")",Character.getFluencyPoints(i));
        }
        var duration = character.periods[i].endmonth - character.periods[i].startmonth + 1;
        for(j=0; j<character.periods[i].interests.length; j++) {
            var dp = character.periods[i].interests[j].ip * duration;
            if (character.periods[i].interests[j].teacher) {
                dp += (Math.round(character.periods[i].interests[j].ip / 3 - 0.3)) * duration;
            }
            addPoints(points,character.periods[i].interests[j].skill,dp);
        }
    }
    // convert to array and sort
    var arr = [];
    for(var s in points) {
        arr.push({skill:s, dp: points[s]});
    }
    // Compute the skill levels
    for(i=0; i<arr.length; i++) {
        arr[i].level = Rules.skilllevel(arr[i].skill,character,arr[i].dp);
    }
    
    
    arr.sort(function(x,y) {
        return y.level - x.level;
    });
    var DOM_table = document.createElement("table");
    var DOM_tbody = document.createElement("tbody");
    var DOM_tr = document.createElement("tr");
    var DOM_th = document.createElement("th");
    DOM_th.colspan = 3;
    var DOM_p = document.createElement("p");
    DOM_p.appendChild(document.createTextNode("As of the month of "+endingMonthText));
    DOM_th.appendChild(DOM_p);
    DOM_tr.appendChild(DOM_th);
    DOM_tbody.appendChild(DOM_tr);
    
    DOM_tr = document.createElement("tr");
    DOM_th = document.createElement("th");
    DOM_th.appendChild(document.createTextNode("Skill Name"));
    DOM_tr.appendChild(DOM_th);
    DOM_th = document.createElement("th");
    DOM_th.appendChild(document.createTextNode("Development Points"));
    DOM_tr.appendChild(DOM_th);
    DOM_th = document.createElement("th");
    DOM_th.appendChild(document.createTextNode("Level"));
    DOM_tr.appendChild(DOM_th);
    DOM_tbody.appendChild(DOM_tr);
    
    for(i=0; i<arr.length; i++) {
        DOM_tr = document.createElement("tr");
        var DOM_td = document.createElement("td");
        DOM_td.appendChild(document.createTextNode(arr[i].skill));
        DOM_tr.appendChild(DOM_td);
        DOM_td = document.createElement("td");
        DOM_td.style = "text-align: right";
        DOM_td.appendChild(document.createTextNode(arr[i].dp));
        DOM_tr.appendChild(DOM_td);
        DOM_td = document.createElement("td");
        DOM_td.style = "text-align: right";
        DOM_td.appendChild(document.createTextNode(arr[i].level));
        DOM_tr.appendChild(DOM_td);
        DOM_tbody.appendChild(DOM_tr);
    }
    DOM_table.appendChild(DOM_tbody);
    
    var skills = document.getElementById("skillsheet");
    Character.deleteAllChildren(skills);
    skills.appendChild(DOM_table);
    
    // And now the JSON needs updating
    character.sheet = { endmonth: character.periods[character.periods.length-1].endmonth,
                        endmonthtext: endingMonthText,
                        skills: arr
                      };
    Character.populateJSON();
};

Character.validateStat = function(name,stat) {
    if (stat === undefined) throw name+" is blank, character has not been updated.";
    var v = Number(String(stat).trim());
    if (isNaN(v)) throw name+" is not a number (it's '"+stat+"'). character has not been updated.";
    if (v < 0) throw name+" is less than zero, character has not been updated.";
    if (v > 20) throw name+" is more than twenty, character has not been updated.";
    return v;
};

Character.createButton = function(name,onclickfunction) {
    var DOM_button = document.createElement("button");
    DOM_button.appendChild(document.createTextNode(name));
    DOM_button.value = name;
    DOM_button.style.width = "100%";
    DOM_button.onclick = onclickfunction;
    return DOM_button;
};

Character.createTextArea = function(id, rows, cols) {
    var DOM_textarea = document.createElement("textarea");
    DOM_textarea.id= id;
    DOM_textarea.rows = rows;
    DOM_textarea.cols = cols;
    if (rows === 1) {
        DOM_textarea.className = "onerow";
    }
    DOM_textarea.appendChild(document.createTextNode(""));
    return DOM_textarea;
};

Character.updateStatus = function(p) {
    var status = document.getElementById("period" + p + "-status");
    var text;
    var valid = false;
    var DOM_b;
    Character.deleteAllChildren(status);
    if (character.periods[p].endmonth === character.periods[p].startmonth-1) {
        status.setAttribute("class", "warning");
        text = "This period is of zero length and will not add any development points to this character's skills.";
        status.appendChild(document.createTextNode(text));
    } else if (character.periods[p].endmonth < character.periods[p].startmonth) {
        status.setAttribute("class", "error");
        text = "This period is of NEGATIVE length and will REMOVE development points from this character's skills.";
        DOM_b = document.createElement("b");
        DOM_b.appendChild(document.createTextNode(text));
        status.appendChild(DOM_b);
    } else if (p === character.periods[p].length-1 && character.periods[p].endMonth !== character.gamemonth-character.birthmonth) {
        status.setAttribute("class", "warning");
        text = "The end month of this last period (" + character.periods[p].endMonthText + ") does not match the game month (" + character.gamemonthtext + "). When the game month is updated, this will reset this end month to the game month.";
        status.appendChild(document.createTextNode(text));
    } else if (character.periods[p].iptotal === 0) {
        status.setAttribute("class", "warning");
        text = "None of the " + character.interestPoints + " interest points available to this character have been used.";
        status.appendChild(document.createTextNode(text));
    } else if (character.periods[p].iptotal < character.interestPoints) {
        status.setAttribute("class", "warning");
        text = "Only " + character.periods[p].iptotal + " of the " + character.interestPoints + " interest points available to this character have been used.";
        status.appendChild(document.createTextNode(text));
    } else if (character.periods[p].iptotal === character.interestPoints) {
        status.setAttribute("class", "");
        text = "All the " + character.interestPoints + " interest points available to this character have been used.";
        valid = true;
        status.appendChild(document.createTextNode(text));
    } else {
        status.setAttribute("class", "error");
        text = "The " + character.periods[p].iptotal + " interest points used by this character are more than the " + character.interestPoints + " available to this character.";
        DOM_b = document.createElement("b");
        DOM_b.appendChild(document.createTextNode(text));
        status.appendChild(DOM_b);
    }
    character.periods[p].statustext = text;
    character.periods[p].valid = valid;
};

Character.populateAll = function() {
    Character.populateStats();
    Character.populateEquipment();
    Character.populatePeriods();
    Character.populateSkills();
};

Character.makePDF = function() {
    var drawSkillTable = function(pdf, title, skills, top, left, cellheight,
            cellwidth) {
        if (skills.length > 45)
            cellheight--; // Cope with Goodgulf :)
        if (skills.length > 50)
            cellheight -= 0.5; // And again

        var skillnamewidth = 81;
        var skilllevelwidth = 60;
        var levelsplit = 112;
        var gutter = ((skills.length > 50) ? 2 : 3);

        var mainleft = left + skillnamewidth + skilllevelwidth;
        var right = mainleft + Rules.adjectives.length * cellwidth;
        var bottom = top + (skills.length + 2) * cellheight;

        pdf.setLineWidth(1);
        pdf.line(mainleft, top, right, top);
        pdf.line(mainleft, top + cellheight, right, top + cellheight);
        pdf.setLineWidth(0.5);
        for (var x = 0; x < Rules.adjectives.length; x++) {
            pdf.line(mainleft + x * cellwidth, top, mainleft + x * cellwidth,
                    bottom);
            pdf.setFontSize(Rules.adjectives[x].length < 12 ? 6 : 4.5); // Special
                                                                        // case
                                                                        // for
                                                                        // "Embarrasing"
            pdf.textInBox(Rules.adjectives[x], {
                align : "center"
            }, mainleft + x * cellwidth + 1, top + cellheight - gutter,
                    mainleft + (x + 1) * cellwidth, top);
        }

        pdf.setLineWidth(1);
        pdf.line(left, top + cellheight, right, top + cellheight);
        pdf.line(left, top + 2 * cellheight, right, top + 2 * cellheight);
        pdf.line(left, bottom, right, bottom);
        pdf.line(left, top + cellheight, left, bottom);
        pdf.line(left + skillnamewidth, top + cellheight,
                left + skillnamewidth, bottom);
        pdf.line(mainleft, top, mainleft, bottom);
        pdf.line(right, top, right, bottom);
        pdf.setFontStyle("bold");
        pdf.setFontSize(7.5);
        pdf.text(left + 1, top + cellheight * 2 - 3, title);
        pdf.text(left + skillnamewidth + 1, top + cellheight * 2 - 3, "Level");
        for (x = 0; x < Rules.adjectives.length; x++) {
            pdf.textInBox(x.toString(), {
                align : "center"
            }, mainleft + x * cellwidth + 1, top + 2 * cellheight - gutter,
                    mainleft + (x + 1) * cellwidth, top + cellheight);
        }

        pdf.setLineWidth(0.5);
        pdf.setFontStyle("normal");
        var y = top + 3 * cellheight;
        for (var s = 0; s < skills.length; s++, y += cellheight) {
            if (s !== skills.length - 1) {
                pdf.line(left, y, right, y);
            }
            pdf.text(left + 1, y - 3, skills[s].skill);
            pdf.textInBox(skills[s].level.toString(), {
                align : "right"
            }, left + skillnamewidth, y - gutter, levelsplit - 1, y - cellheight + 1);
            pdf.textInBox(Rules.adj(skills[s].level), {
                align : "left"
            }, levelsplit + 1, y - gutter, mainleft, y - cellheight + 1);
            var perf = Rules.performance(skills[s].level);
            for (x = 0; x < Rules.adjectives.length; x++) {
                pdf.textInBox(perf[x], {
                    align : "center"
                }, mainleft + x * cellwidth + 1, y - gutter, mainleft + (x + 1) * cellwidth, y - cellheight + 1);
            }
        }
    };

    var pdf = new jsPDF("landscape", "pt", "a4");
    pdf.setProperties({
        title : character.name,
        keywords : "generated, linrodeth, character"
    });

    var namebottom = 18;
    var nameleft = 0;
    var nameright = 842;
    var nametop = 0;
    var left = 12;
    var cellwidth = 32;
    var cellheight = 10;
    var chartext = "Rules Version " + character.rulesVersion + ", Implementation Version " + Rules.version + ", Character version " + character.characterVersion + ", Game Date:" + character.gameday + ", (last edit time " + character.lastEditDate + ", PDF created " + (new Date()).toLocaleString() + ")";

    pdf.setFontSize(18);
    pdf.textInBox(character.name, {
        align : "center"
    }, nameleft, namebottom, nameright, nametop);

    // Need to Fake up a skills array for the attributes
    var attributesAsSkills = [];
    for (var stat = 0; stat < Character.allstats.length; stat++) {
        attributesAsSkills.push({
            "skill" : Character.longstats[stat],
            "level" : character[Character.allstats[stat]]
        });
    }
    drawSkillTable(pdf, "Attribute", attributesAsSkills, 36, left, cellheight,
            cellwidth);

    var drawTable = function(pdf, content, widths, aligns, left, top,
            cellheight, borderweights) {
        var totalwidth = widths.reduce(function(x, y) {
            return x + y;
        }, 0);
        pdf.setLineWidth(1);
        pdf.rect(left, top, totalwidth, content.length * cellheight);
        var offset = left + widths[0];
        var cell;
        for (cell = 1; cell < widths.length; cell++) {
            if (borderweights && borderweights[cell])
                pdf.setLineWidth(borderweights[cell]);
            pdf.line(offset, top, offset, top + content.length * cellheight);
            offset += widths[cell];
        }
        pdf.setFontStyle("bold"); // Make top line bold
        pdf.setLineWidth(1);
        y = top + cellheight;
        for (var line = 0; line < content.length; line++, y += cellheight) {
            if (line !== content.length - 1) {
                pdf.line(left, y, left + totalwidth, y);
            }
            offset = left;
            for (cell = 0; cell < aligns.length; cell++) {
                pdf.textInBox(content[line][cell], {
                    align : aligns[cell]
                }, offset + 1, y - 3, offset + widths[cell], y - cellheight);
                offset += widths[cell];
            }
            if (line === 0) {
                pdf.setFontStyle("normal");
                pdf.setLineWidth(0.5);
            }
        }
    };

    var i,w,x,weapon;
    var content;
    content = [];
    var widths;
    widths = [ 81, 30, 30 ];
    var aligns;
    aligns = [ "left", "center", "center" ];
    var title;
    title = [ "Weapon", "Area", "Hits" ];
    for (x = 0; x < Rules.adjectives.length; x++) {
        title.push(x.toString());
        widths.push(cellwidth);
        aligns.push("center");
    }
    content.push(title);
    for (w = 0; w < character.favouredWeapons.length; w++) {
        weapon = character.favouredWeapons[w];
        if (weapon.ranged) {
            var area = weapon.area || "";
            var hits = weapon.hits || "";
            var ranges = [ weapon.name, area, hits ];
            ranges = ranges.concat(weapon.ranges);
            for (i = 0; i < ranges.length; i++) {
                ranges[i] = (ranges[i] ? ranges[i].toString() : "");
            }
            content.push(ranges);
        }
    }
    pdf.text(left + 3, 151, "Difficulty to hit at range");
    drawTable(pdf, content, widths, aligns, left, 154, cellheight, [ , 1, 0.5,
            1, 0.5 ]);

    content = [];
    widths = [ 81, 60 ];
    aligns = [ "left", "center" ];
    title = [ "Weapon", "Damage" ];
    for (x = 0; x < Rules.adjectives.length; x++) {
        title.push(x.toString());
        widths.push(cellwidth);
        aligns.push("center");
    }
    content.push(title);
    var line;
    line = [ "Unarmed", "1" ];
    line = line.concat(Rules.damageTable[1]);
    content.push(line);
    for (w = 0; w < character.favouredWeapons.length; w++) {
        weapon = character.favouredWeapons[w];
        line = [ weapon.name, weapon.damage.toString() ];
        line = line.concat(Rules.damageTable[weapon.damage]);
        content.push(line);
    }
    drawTable(pdf, content, widths, aligns, left, 210, cellheight, [ , 1, 1,
            0.5 ]);

    content = [];
    content.push([ "Situation", "Mod", "Example for cooking" ]);
    content.push([ "Excellent", "+5", "Staffed restaurant kitchen" ]);
    content.push([ "Very good", "+3", "Skilled help" ]);
    content.push([ "Good", "+1", "Normal kitchen, some help" ]);
    content.push([ "Normal", "", "Normal kitchen" ]);
    content.push([ "Poor", "-1", "No spices or herbs available" ]);
    content.push([ "Very Poor", "-3", "Open fire, few ingredients" ]);
    content.push([ "Terrible", "-5", "No fire" ]);
    drawTable(pdf, content, [ 41, 120 - 86, 225 - 120 ], [ "left", "center",
            "left" ], 45, 324, cellheight);

    var modwidths = [ 90, 36 ];

    content = [];
    content.push([ "Melee situation", "Modifier" ]);
    content.push([ "Defending only", "+3" ]);
    content.push([ "At Opponents side", "+1" ]);
    content.push([ "Behind Opponent", "+2" ]);
    content.push([ "Unarmed versus blunt", "-1" ]);
    content.push([ "Unarmed versus blade", "-4" ]);
    content.push([ "Shorter weapon", "-1" ]);
    drawTable(pdf, content, modwidths, [ "left", "center" ], 252, 324,
            cellheight);

    content = [];
    content.push([ "Location Hit", "Melee", "Ranged" ]);
    content.push([ "Head", "91-00", "96-00" ]);
    content.push([ "Left Arm", "81-90", "86-95" ]);
    content.push([ "Right Arm", "71-80", "76-85" ]);
    content.push([ "Chest", "45-70", "51-75" ]);
    content.push([ "Abdomen", "33-44", "33-50" ]);
    content.push([ "Left Leg", "17-32", "17-32" ]);
    content.push([ "Right Leg", "01-16", "01-16" ]);
    drawTable(pdf, content, [ 64, 32, 32 ], [ "left", "center", "center" ],
            405, 324, cellheight);

    content = [];
    content.push([ "Ranged situation", "Modifier" ]);
    content.push([ "Aimed Shot (2 rounds)", "+2" ]);
    content.push([ "Walking target", "-2" ]);
    content.push([ "Jogging target", "-3" ]);
    content.push([ "Running target", "-4" ]);
    content.push([ "Dodging target", "-5" ]);
    content.push([ "Light cover", "-1" ]);
    content.push([ "Medium cover", "-3" ]);
    content.push([ "Heavy cover", "-5" ]);
    drawTable(pdf, content, modwidths, [ "left", "center" ], 252, 400,
            cellheight);

    content = [];
    content.push([ "Modifiers for previous round", "Modifier" ]);
    content.push([ "Minor defeat last round", "-1" ]);
    content.push([ "Clear defect last round", "-2" ]);
    content.push([ "Major defect last round", "-3" ]);
    content.push([ "Devastating defect last round", "-4" ]);
    drawTable(pdf, content, [ 148, 32 ], [ "left", "center" ], 45, 414,
            cellheight);

    content = [];
    content.push([ "Gap", "Type of win", "Wound Location" ]);
    content.push([ "1-3", "Minor", "None" ]);
    content.push([ "4-6", "Clear", "Random" ]);
    content.push([ "7-9", "Major", "Move 1 place, +2 damage" ]);
    content.push([ "10+", "Devastating", "Move 2 places, +5 damage" ]);
    drawTable(pdf, content, [ 32, 120 - left - 32, 225 - 120 ], [ "left",
            "left", "left" ], left, 477 + cellheight, cellheight);
    pdf.setLineWidth(1);
    pdf.setFontStyle("bold");
    pdf.line(left, 477 + cellheight, left, 477);
    pdf.line(left, 477, 225, 477);
    pdf.line(225, 477, 225, 477 + cellheight);
    pdf.text(left, 477 + cellheight - 3, "Performance Difference");

    content = [];
    content.push([ "Type of encumbrance", "Modifier" ]);
    content.push([ "Light encumbrance", "" ]);
    content.push([ "Medium encumbrance", "-1" ]);
    content.push([ "Heavy encumbrance", "-3" ]);
    drawTable(pdf, content, modwidths, [ "left", "center" ], 252, 497,
            cellheight);

    content = [];
    content.push([ "Movement", "Speed" ]);
    content.push([ "Walk", "1" ]);
    content.push([ "Jog", "2" ]);
    content.push([ "Sprint", "4" ]);
    drawTable(pdf, content, [ 64, 32 ], [ "left", "center" ], 405, 497,
            cellheight);

    pdf.setLineWidth(1);
    pdf.rect(569, 324, 8 * cellwidth, 8 * cellheight);
    pdf.line(569, 324 + cellheight, 569 + 2 * cellwidth, 324 + cellheight);
    pdf.line(569 + 2 * cellwidth, 324, 569 + 2 * cellwidth, 324 + cellheight);
    pdf.setLineWidth(0.5);
    for (i = 1; i < 8; i++) {
        pdf.line(569 + i * cellwidth, 324, 569 + i * cellwidth,
                324 + 8 * cellheight);
        pdf.line(569, 324 + i * cellheight, 569 + 8 * cellwidth, 324 + i * cellheight);
    }
    pdf.text(570, 324 + cellheight - 3, "Morale");
    pdf
            .text(570 + cellwidth, 324 + cellheight - 3, character.morale
                    .toString());

    content = [];
    content.push([ "Area", "Armour", "WP", "Armour" ]);
    var locline = function(name, location) {
        var containsShieldLocation = function(location) {
            return function(x) {
                return x === location;
            };
        };

        var armour = character.armour.name;
        var points = character.armour.value;
        if (character.shieldLocations.some(containsShieldLocation(location))) {
            armour += " + " + character.shield.name;
            points += character.shield.value;
        }
        return [ name, armour, character.woundPoints[location].toString(),
                points.toString() ];
    };
    content.push(locline("Head", "head"));
    content.push(locline("Shield Arm", "shieldArm"));
    content.push(locline("Sword Arm", "swordArm"));
    content.push(locline("Chest", "chest"));
    content.push(locline("Abdomen", "abdomen"));
    content.push(locline("Right Leg", "rightLeg"));
    content.push(locline("Left Leg", "leftLeg"));
    drawTable(pdf, content, [ 64, 100, 32, 32 ], [ "left", "left", "center",
            "center" ], 405, 410, cellheight, [ 1, 1, 1, 0.5, 1, 0.5 ]);
    pdf.setLineWidth(1);
    pdf.rect(633, 420, 6 * cellwidth, 7 * cellheight);
    pdf.setLineWidth(0.5);
    for (i = 1; i < 7; i++) {
        pdf.line(633, 420 + i * cellheight, 633 + 6 * cellwidth, 420 + i * cellheight);
    }
    for (i = 1; i < 6; i++) {
        pdf.line(633 + i * cellwidth, 420, 633 + i * cellwidth, 420 + 7 * cellheight);
    }

    pdf.setFontSize(8);
    pdf.text(left, 576, chartext);

    pdf.addPage();
    pdf.setFontSize(18);
    pdf.textInBox(character.name, {
        align : "center"
    }, nameleft, namebottom, nameright, nametop);

    var untrained = [];
    untrained.push({
        "skill" : "All untrained skills",
        "level" : 0
    });
    drawSkillTable(pdf, "Skill", untrained, 27, left, cellheight, cellwidth);
    drawSkillTable(pdf, "Skill", character.sheet.skills, 72, left, cellheight,
            cellwidth);

    pdf.setFontSize(8);
    pdf.text(left, 576, chartext);

    return pdf;
};

Character.initPage = function() {
    // Initialise the character, either with data from an embedded JSON or a
    // completely new one
    if (document.getElementById("character-data") && document.getElementById("character-data").text && document.getElementById("character-data").text.length > 10) {
        Character.updateWithNewJSON(
                document.getElementById("character-data").text, false);
    } else {
        character = Rules.newCharacter();
        character.schemaVersion = 11;
        character.characterVersion = 0;
        character.lastEditDate = (new Date(Date.now())).toLocaleString();
        character.rulesVersion = "2014a";
        character.genre = new Genre(options.default_genre);
        character.name = "Nobody Noname";
        character.playerName = document.getElementById("playernametextarea").textContent;
        character.description = "An outstanding character whose deeds are described herein and whose heroic future is still to come.";
        character.statslock = false;
        character.morale = Rules.startingMorale(character);
        character.woundPoints = Rules.woundPoints(character);
        character.interestPoints = Rules.interestPoints(character);
        character.birthtime = Date.now() - 32 * 365.25 * 24 * 60 * 60 * 1000;
        character.birthmonth = character.genre.asMonth(character.birthtime);
        character.birthmonthtext = character.genre.dateMonthAsText(character.birthmonth);
        character.birthday = character.genre.dayMonthYearAsText(character.birthtime, character.birthmonth);
        character.gametime = options.gametime;
        character.gamemonth = character.genre.asMonth(character.gametime);
        character.gamemonthtext = character.genre.dateMonthAsText(character.gamemonth);
        character.gameday = character.genre.dayMonthYearAsText(character.gametime, character.gamemonth);
        character.periods = [ {
            title : "Childhood",
            description : "The early years discovering the world and your place in it",
            startmonth : 60,
            startmonthtext : character.genre.dateMonthAsText(character.birthmonth + 60),
            endmonth : character.gamemonth - character.birthmonth,
            endmonthtext : character.gamemonthtext,
            childhoodLanguage : "Athionic",
            interests : [],
            iptotal : 0,
            statustext : "None of the " + character.interestPoints + " interest points available to this character have been used.",
            valid : false
        } ];
        character.newSkills = [];
        character.armour = Rules.armour[0];
        character.shield = Rules.shields[0];
        character.shieldLocations = [];
        character.favouredWeapons = [];
        character.syncWithGlobalDate = true;
    }
    Character.edittableStats = false; // Note the big C on the front of this,
                                        // this property is not part of the
                                        // character.

    // Set up all the non-Period buttons
    Character.setupButtons();

    document.getElementById("JSONsavefile").onclick = Character
            .returnFalse(function() {
                Character.incrementVersion();
                var blob = new Blob(
                        [ JSON.stringify(character, undefined, 2) ], {
                            type : "text/plain;charset=utf-8"
                        });
                saveAs(blob, character.name + ".txt");
            });

    document.getElementById("savePDF").onclick = Character
            .returnFalse(function() {
                var blob = Character.makePDF().output('blob');
                saveAs(blob, character.name + ".pdf");
            });
};

Character.updateWithNewJSON = function(json,populate) {
    try {
        var i;
        var newCharacter = JSON.parse(json);
        for(i=0; i<Character.allstats.length; i++) {
            newCharacter[Character.allstats[i]] = Character.validateStat(Character.allstats[i],newCharacter[Character.allstats[i]]);
        }
        character = newCharacter;

        if (!character.genre) {
            // Need to guess! Base off of the childhood language in the first period
            if (character.periods[0].childhoodLanguage === "Athionic") {
                character.genre = new Genre("Linrodeth");
            } else if (character.periods[0].childhoodLanguage === "Latin") {
                character.genre = new Genre("Luna Romana");
            } else if (character.periods[0].childhoodLanguage === "English") {
                character.genre = new Genre("Contemporary");
            } else {
                character.genre = new Genre("Linrodeth");
            }
        } else {
            character.genre = new Genre(character.genre);
        }

        character.morale = Rules.startingMorale(character);
        character.woundPoints = Rules.woundPoints(character);
        character.interestPoints = Rules.interestPoints(character);
        if (!character.schemaVersion) {
            character.schemaVersion = 1;
            character.characterVersion = 1e9; // Large offset to make it obvious
            character.lastEditDate = (new Date(Date.now())).toLocaleString();
            character.rulesVersion = "2014a";
            character.description = "An outstanding character whose deeds are described herein.";
            character.gametime = character.birthtime + character.periods[character.periods.length-1].endmonth*30.4375*24*60*60*1000; // Approx
            character.gameday = character.genre.dayMonthYearAsText(character.gametime,character.birthmonth);
            character.gamemonth = character.periods[character.periods.length-1].endmonth + character.birthmonth;
            character.gamemonthtext = character.genre.dateMonthAsText(character.gamemonth);
            character.periods[character.periods.length-1].endMonth = character.gamemonth;
            character.periods[character.periods.length-1].endMonthText = character.gamemonthtext;
            character.statslock = true;
            Character.updateMonths();
        }
        Rules.resetAddedSkills();
        Character.resetOtherSkillsList();
        
        // Schema update processing
        var name;
        if (character.schemaVersion === 1) {
            character.schemaVersion = 2;
        } else {
            for(i=0; i<character.newSkills.length; i++) {
                for(name in character.newSkills[i]) {
                    Rules.addSkill(name,character.newSkills[i][name]);
                }
            }
        }
        // Rebuild the newSkills list, (to remove duplicates and to ensure the added skills remain even when building lots of characters from the same instance
        character.newSkills = [];
        for(name in Rules.skilllist["Added skills"]) {
            Character.addOtherSkill(name,Rules.skilllist["Added skills"][name]);
        }
        
        // Continue with the schema updates
        if (character.schemaVersion === 2) {
            character.schemaVersion = 3;
            character.gameday = character.genre.dayMonthYearAsText(character.gametime,character.birthmonth);
        }
        if (character.schemaVersion === 3) {
            character.schemaVersion = 4;
            character.armour = Rules.armour[0];
            character.shield = Rules.shields[0];
            character.shieldLocations = [];
            character.favouredWeapons = [];
        }
        if (character.schemaVersion === 4) {
            character.schemaVersion = 5;
            character.gameday = character.genre.dayMonthYearAsText(character.gametime,character.birthmonth);
        }
        if (character.schemaVersion === 5) {
            character.schemaVersion = 6;
            character.birthday = character.genre.dayMonthYearAsText(character.birthtime,character.birthmonth);
            character.birthmonth = character.genre.asMonth(character.birthtime);
            character.birthmonthtext = character.genre.dateMonthAsText(character.birthmonth);
            character.gameday = character.genre.dayMonthYearAsText(character.gametime,character.gamemonth);
            character.gamemonth = character.genre.asMonth(character.gametime);
            character.gamemonthtext = character.genre.dateMonthAsText(character.gamemonth);
        }
        if (character.schemaVersion === 6) {
            character.schemaVersion = 7;
            character.playerName = document.getElementById("playernametextarea").textContent;
        }
        if (character.schemaVersion < 9) {
            character.schemaVersion = 9;
            if (character.genre.equals(new Genre('Linrodeth'))) {
                character.birthmonth--;
                character.gamemonth--;
            }
            for(i=0; i<character.periods.length; i++) {
                character.periods[i].startmonthtext = character.genre.dateMonthAsText(character.periods[i].startmonth+character.birthmonth);
                character.periods[i].endmonthtext = character.genre.dateMonthAsText(character.periods[i].endmonth+character.birthmonth);
            }
            character.birthmonthtext = character.genre.dateMonthAsText(character.birthmonth);
            character.gamemonthtext = character.genre.dateMonthAsText(character.gamemonth);
        }
        if (character.schemaVersion === 9) {
            character.schemaVersion = 10;
            character.morale = Rules.startingMorale(character); // rounding change
        }
        if (character.schemaVersion == 10) {
            character.schemaVersion = 11;
            character.syncWithGlobalDate = (character.gametime > 1200000000000);
        }
        
        // Data resets - resolve skills with 
        var ampregexp = new RegExp('&amp;', 'g');
        for(i=0; i<character.periods.length; i++) {
            for(var j=0; j<character.periods[i].interests.length; j++) {
                character.periods[i].interests[j].skill = character.periods[i].interests[j].skill.replace(ampregexp,'&');
            }
        }
        
        // Global updates
        if (character.syncWithGlobalDate) {
            character.gametime = options.gametime;
            character.gameday = character.genre.dayMonthYearAsText(character.gametime,character.gamemonth);
            character.gamemonth = character.genre.asMonth(character.gametime);
            character.gamemonthtext = character.genre.dateMonthAsText(character.gamemonth);
            character.periods[character.periods.length-1].endmonth = character.gamemonth - character.birthmonth;
            character.periods[character.periods.length-1].endmonthtext = character.gamemonthtext;
        }
        
        if (populate) {
            Character.populateAll();
        }
    } catch(e) {
        alert(e);
    }
};
