//
// © Copyright 2014 David Vines
//
// Creative Commons License
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
//
// Big 'C' Character is used to namespace the functions
//
// little 'c' character contains the character being created.
Character.populateStats = function() {
    var i;
    Character.setTextArea("nametextarea",character.name);
    Character.setTextArea("playernametextarea",character.playerName);
    Character.setOption(document.getElementById("genre"),character.genre.toString());
    for(i=0; i<Character.allstats.length; i++) {
        if (Character.edittableStats) {
            Character.setTextArea("stats-"+Character.allstats[i],character[Character.allstats[i]]);
    } else {
        Character.setIdText("stats-"+Character.allstats[i],character[Character.allstats[i]]);
        }
    }
    Character.setIdText("stats-morale",character.morale);
    Character.setIdText("stats-ip",character.interestPoints);

    Character.setTextArea("characterdescriptiontextarea",character.description);

    Character.setIdText("stats-birthday",character.genre.dayMonthYearAsText(character.birthtime,character.birthmonth));
    Character.setIdText("stats-gameday",character.genre.dayMonthYearAsText(character.gametime,character.gamemonth));

    if (character.statslock) {
        Character.deleteElement("birthdayminus1year");
        Character.deleteElement("birthdayminus1month");
        Character.deleteElement("birthdayminus1day");
        Character.deleteElement("birthdayplus1day");
        Character.deleteElement("birthdayplus1month");
        Character.deleteElement("birthdayplus1year");
        Character.deleteElement("lockform");
        for(i=0; i<Character.allstats.length; i++) {
            Character.deleteElement(Character.allstats[i]+"swap");
        }
    }

    document.getElementById("gamedatesync").checked = character.syncWithGlobalDate;
    document.getElementById("gamedatesync").defaultChecked = character.syncWithGlobalDate;
    document.getElementById("gamedayminus1year").disabled = character.syncWithGlobalDate;
    document.getElementById("gamedayminus1month").disabled = character.syncWithGlobalDate;
    document.getElementById("gamedayminus1day").disabled = character.syncWithGlobalDate;
    document.getElementById("gamedayplus1day").disabled = character.syncWithGlobalDate;
    document.getElementById("gamedayplus1month").disabled = character.syncWithGlobalDate;
    document.getElementById("gamedayplus1year").disabled = character.syncWithGlobalDate;
};

Character.updateNode = function(node,i) {
    if (node.nodeType === Node.ELEMENT_NODE) {
        node.id = node.id.replace("@@",i.toString());
        if (node.name !== undefined) {
            node.name = node.name.replace("@@",i.toString());
        }
        for(var j = 0; j < node.childNodes.length; j++) {
            Character.updateNode(node.childNodes[j],i);
        }
    }
};

Character.updateForm = function(node,i) {
    if (node.nodeType === Node.ELEMENT_NODE) {
        node.id = node.id.replace("-interest0-","-interest"+i+"-");
        if (node.name !== undefined) {
            node.name = node.name.replace("-interest0-","-interest"+i+"-");
        }
        for(var j = 0; j < node.childNodes.length; j++) {
            Character.updateForm(node.childNodes[j],i);
        }
    }
};

Character.updateEndDate = function(p,delta) {
    return Character.returnFalse(function() {
        character.periods[p].endmonth += delta;
        character.periods[p].endmonthtext = character.genre.dateMonthAsText(character.periods[p].endmonth+character.birthmonth);
        if (character.periods[p+1]) {
            // Update start of next period
            character.periods[p+1].startmonth = character.periods[p].endmonth+1;
            character.periods[p+1].startmonthtext = character.genre.dateMonthAsText(character.periods[p+1].startmonth+character.birthmonth);
            // Future period now at zero or negative length?, if so we need to bump its end month too
            if (character.periods[p+1].startmonth > character.periods[p+1].endmonth) {
                    // By just enough to make it one month in length
                    Character.updateEndDate(p+1,character.periods[p+1].startmonth - character.periods[p+1].endmonth + 1);
            }
        }
        Character.incrementVersion();
        Character.populatePeriods();
        Character.populateSkills();
      });
};

Character.updateFluency = function(p,fc) {
    return Character.returnFalse(function() {
        character.periods[p].childhoodLanguage = fc[fc.selectedIndex].text;
        Character.incrementVersion();
        Character.populateSkills();
    });
};

Character.updatePoints = function(p,i,delta) {
    return Character.returnFalse(function() {
        character.periods[p].interests[i].ip += delta;
        character.periods[p].iptotal += delta;
        Character.updateSkillRow(p,i);
        Character.updateSkillRowButtons(p);
        Character.updateStatus(p);
        Character.incrementVersion();
        Character.populateSkills();
    });
};

Character.updateTeacher = function(p,i,sc) {
    return Character.returnFalse(function() {
        character.periods[p].interests[i].teacher = sc.checked;
        Character.incrementVersion();
        Character.populateSkills();
    });
};

Character.updateSkillRowButtons = function(p) {
    for(var i=0; i<character.periods[p].interests.length; i++) {
        interestup = document.getElementById("period"+p+"-interest"+i+"-ipup");
        interestdown = document.getElementById("period"+p+"-interest"+i+"-ipdown");
        interestup.disabled = (character.periods[p].iptotal >= character.interestPoints);
        interestdown.disabled = (character.periods[p].interests[i].ip < 1);
    }
};

Character.updateSkillRow = function(p,interest) {
    var interestteacher = document.getElementById("period"+p+"-interest"+interest+"-teacher");
    Character.setIdText("period"+p+"-interest"+interest+"-skill",character.periods[p].interests[interest].skill);
    Character.setIdText("period"+p+"-interest"+interest+"-points",character.periods[p].interests[interest].ip);
    Character.setChecked(interestteacher,character.periods[p].interests[interest].teacher);
};

Character.deleteInterest = function(p,interest) {
    return Character.returnFalse(function() {
        character.periods[p].iptotal -= character.periods[p].interests[interest].ip;
        character.periods[p].interests.splice(interest,1);
        for(var i=interest; i<character.periods[p].interests.length; i++) {
            Character.updateSkillRow(p,i);
        }
        var row = document.getElementById("period"+p+"-interest"+character.periods[p].interests.length);
        Character.deleteElement(row);
        Character.updateSkillRowButtons(p);
        Character.updateStatus(p);
        Character.incrementVersion();
        Character.populateSkills();
    });
};

Character.addSkillRow = function(p,interest) {
    Character.updateSkillRow(p,interest);
    var interestup = document.getElementById("period"+p+"-interest"+interest+"-ipup");
    var interestdown = document.getElementById("period"+p+"-interest"+interest+"-ipdown");
    var interestteacher = document.getElementById("period"+p+"-interest"+interest+"-teacher");
    var interestdelete = document.getElementById("period"+p+"-interest"+interest+"-delete");
    interestup.onclick = Character.updatePoints(p,interest,1);
    interestdown.onclick = Character.updatePoints(p,interest,-1);
    interestteacher.onchange = Character.updateTeacher(p,interest,interestteacher);
    interestdelete.onclick = Character.deleteInterest(p,interest);
};

Character.addInterestKnowingPeriod = function(p,table,skillname) {
    var adding = true;
    for(var i=0; i<character.periods[p].interests.length; i++) {
        if (character.periods[p].interests[i].skill == skillname) {
            character.periods[p].iptotal += 1;
            character.periods[p].interests[i].ip += 1;
            Character.updateSkillRow(p,i);
            adding = false;
        }
    }
    if (adding) {
        var next = character.periods[p].interests.length;
        character.periods[p].interests.push(JSON.parse(JSON.stringify(Character.templates.interest)));
        character.periods[p].interests[next].skill = skillname;
        character.periods[p].interests[next].ip = 1;
        character.periods[p].iptotal += 1;
        Character.addInterestRow(p,table,next);
    }
    Character.updateSkillRowButtons(p);
    Character.populateSkills();
    Character.populateJSON();
};

Character.populatePeriodTitle = function(p,li) {
    return Character.returnFalse(function() {
        var newtitle = document.getElementById("period"+p+"-titletextarea").value;
        character.periods[p].title = newtitle;
        Character.setIdText("sheetnav"+li,newtitle);
        document.getElementById("sheetnav"+li).title = newtitle;
        Character.incrementVersion();
        Character.populateJSON();
    });
};

Character.populatePeriodDescription = function(p) {
    return Character.returnFalse(function() {
        character.periods[p].description =  document.getElementById("period"+p+"-descriptiontextarea").value;
        Character.incrementVersion();
        Character.populateJSON();
    });
};

Character.populatePeriods = function() {    
    // Delete the old periods
    var i = 0;
    var period = document.getElementById("period"+i);
    while(period) {
        Character.deleteElement(period);
        i++;
        period = document.getElementById("period"+i);
    }

    // And build new ones
    var periods = document.getElementById("periods");
    var template = document.getElementById("period@@");
    var agemonths = character.gamemonth - character.birthmonth;

    for(i = 0; i < character.periods.length; i++) {
        var newNode = template.cloneNode(true);
        newNode.className = "tabbertab";
        newNode.title = character.periods[i].title;
        Character.updateNode(newNode,i);
        periods.parentNode.insertBefore(newNode,periods);
        
        // Header fields
        Character.setTextArea("period"+i+"-titletextarea",character.periods[i].title);
        Character.setTextArea("period"+i+"-descriptiontextarea",character.periods[i].description);
        
        // dates
        Character.setIdText("period"+i+"-startdate","Age: "+Character.durationLength(character.periods[i].startmonth),character.genre.dateMonthAsText(character.periods[i].startmonth+character.birthmonth));
        Character.setIdText("period"+i+"-enddate","Age: "+Character.durationLength(character.periods[i].endmonth),character.genre.dateMonthAsText(character.periods[i].endmonth+character.birthmonth));
        Character.setIdText("period"+i+"-enddatelength"," A period of "+Character.durationLength(character.periods[i].endmonth - character.periods[i].startmonth  + 1));
        document.getElementById("period"+i+"-titletextarea").onchange = Character.populatePeriodTitle(i,i+3);
        document.getElementById("period"+i+"-descriptiontextarea").onchange = Character.populatePeriodDescription(i);
        
        document.getElementById("period"+i+"-enddateminus1year").onclick = Character.updateEndDate(i,-12);
        document.getElementById("period"+i+"-enddateminus1month").onclick = Character.updateEndDate(i,-1);
        document.getElementById("period"+i+"-enddateplus1month").onclick = Character.updateEndDate(i,1);
        document.getElementById("period"+i+"-enddateplus1year").onclick = Character.updateEndDate(i,12);
        
        Character.populatePeriodMonths(i);
        
        // childhood fluency
        if (character.periods[i].startmonth > 180) {
            Character.deleteElement(document.getElementById("period"+i+"-fluencyheader"));
            Character.deleteElement(document.getElementById("period"+i+"-fluencycontent"));
        } else {
            Character.updateFluencyPoints(i);
            var fluencyChoice = document.getElementById("period"+i+"-childhoodfluencylanguage");
            Character.setOption(fluencyChoice,character.periods[i].childhoodLanguage);
            fluencyChoice.onchange = Character.updateFluency(i,fluencyChoice);
        }
        
        // Interests
        var table = document.getElementById("period"+i+"-details").firstChild.nextSibling;
        for(var interest=0; interest < character.periods[i].interests.length; interest++) {
            Character.addInterestRow(i,table,interest);
        }
        Character.updateSkillRowButtons(i);
        
        Character.updateStatus(i);
    }
    
    Character.updateTabs();
};

Character.populateJSON = function() {
    Character.setTextArea("JSONtextarea",JSON.stringify(character,undefined,2));
};

Character.birthdaydelta = function(delta) {
    return Character.returnFalse(function() {
        character.birthtime += delta;
        character.birthmonth = character.genre.asMonth(character.birthtime);
        character.birthmonthtext = character.genre.dateMonthAsText(character.birthmonth);
        character.birthday = character.genre.dayMonthYearAsText(character.birthtime,character.birthmonth);
        character.periods[character.periods.length-1].endmonth = character.gamemonth - character.birthmonth;
        for(var i=0; i<character.periods.length; i++) {
            character.periods[i].startmonthtext = character.genre.dateMonthAsText(character.periods[i].startmonth+character.birthmonth);
            character.periods[i].endmonthtext = character.genre.dateMonthAsText(character.periods[i].endmonth+character.birthmonth);
        }
        Character.incrementVersion();
        Character.populateAll(); // all the dates will have shifted in sympathy
    });
} ;

Character.gamedaydelta = function(delta) {
    return Character.returnFalse(function() {
        character.gametime += delta;
        character.gamemonth = character.genre.asMonth(character.gametime);
        character.gamemonthtext = character.genre.dateMonthAsText(character.gamemonth);
        character.gameday = character.genre.dayMonthYearAsText(character.gametime,character.gamemonth);
        character.periods[character.periods.length-1].endmonth = character.gamemonth - character.birthmonth;
        character.periods[character.periods.length-1].endmonthtext = character.gamemonthtext;
        Character.incrementVersion();
        Character.populateAll(); // The last period will have changed length (possibly to a negative length)
    });
} ;

Character.updateDerivedStats = function() {
    character.morale = Rules.startingMorale(character);
    character.woundPoints = Rules.woundPoints(character);
    character.interestPoints = Rules.interestPoints(character);
    for(var p=0; p<character.periods.length; p++) {
        character.periods[p].iptotal = 0;
        for(var i=0; i<character.periods[p].interests.length; i++) {
            character.periods[p].iptotal += character.periods[p].interests[i].ip;
        }
        Character.updateStatus(p);
    }
    Character.populateAll();
};

Character.statSwap = function(stat,sc) {
    return  Character.returnFalse(function() {
        var option = sc[sc.selectedIndex].text;
        if (option === "") {
            return;
        }
        var shorthand = { "Strength": "str",
                          "Dexterity": "dex",
                          "Agility": "agi",
                          "Constitution": "con",
                          "Appearance": "app",
                          "Intellect": "int",
                          "Wit": "wit",
                          "Will": "will"
                        };
        that = shorthand[option];
        var other = character[that];
        character[that] = character[stat];
        character[stat] = other;
        Character.updateDerivedStats();
        Character.incrementVersion();
        sc.selectedIndex = 0;
    });
};

Character.resetOtherSkillsList = function() {
    var DOM_h4 = document.getElementById("skills-category-"+encodeURI("Added skills"));
    if (DOM_h4) {
        var DOM_table = DOM_h4.nextSibling;
        var DOM_tbody = DOM_table.firstChild;
        Character.deleteAllChildren(DOM_tbody);
    }
};

Character.addOtherSkill = function(name,stats) {
    if (name !== "") {
        Rules.addSkill(name,stats);
        
        if (character.newSkills.every(function(x) { return x[name] === undefined; })) {
            var newSkills = {};
            newSkills[name] = stats;
            character.newSkills.push(newSkills);
        }
        
        // Finally add a button for it - if the table for the buttons exists
        var DOM_h4 = document.getElementById("skills-category-"+encodeURI("Added skills"));
        var DOM_table, DOM_tbody, DOM_tr, DOM_td;
        if (DOM_h4) {
            DOM_table = DOM_h4.nextSibling;
            DOM_tbody = DOM_table.firstChild;
            if (!DOM_tbody.lastChild || DOM_tbody.lastChild.lastChild.lastChild) {
                // Empty space on last row
                DOM_tr = document.createElement("tr");
                DOM_td = document.createElement("td");
                DOM_td.appendChild(Character.createButton(name,Character.addInterest(name)));
                DOM_tr.appendChild (DOM_td);
                DOM_td = document.createElement("td");
                DOM_tr.appendChild(DOM_td);
                DOM_tbody.appendChild(DOM_tr);
            } else {
                DOM_td = DOM_tbody.lastChild.lastChild;
                DOM_td.appendChild(Character.createButton(name,Character.addInterest(name)));
            }
        }
    }
    
    // And give the character the skill if they're on a period
    Character.addInterest(name)();
};

Character.init = function() {
    // Copy across the form fields to the ones actually sent (in the hidden diff)
    Character.setTextArea("djv_nametextarea",character.name);
    Character.setTextArea("djv_genre",character.genre.toString());
    Character.setTextArea("djv_playernametextarea",character.playerName);

    
    // Populate the skill list div with buttons
    var skilllist = document.getElementById("skilllist").parentNode;
    var childhoodfluency = document.getElementById("period@@-childhoodfluencylanguage");
    var DOM_h4, DOM_form, DOM_table, DOM_tbody, DOM_tr, DOM_td, DOM_button, DOM_option;
    for(var category in Rules.skilllist) {
        DOM_h4 = document.createElement("h4");
        DOM_h4.appendChild(document.createTextNode(category));
        DOM_h4.id = "skills-category-"+encodeURI(category); // to encode the blanks
        skilllist.appendChild(DOM_h4);
        DOM_table = document.createElement("table");
        DOM_table.style.width = "100%";
        DOM_table.style.tableLayout = "fixed";
        DOM_tbody = document.createElement("tbody");
        DOM_tr = undefined;
        for(var skill in Rules.skilllist[category]) {
            DOM_td = document.createElement("td");
            DOM_td.appendChild(Character.createButton(skill,Character.addInterest(skill)));
            // Flip-flop the cells into rows so that we get two columns
            if (DOM_tr) {
                DOM_tr.appendChild(DOM_td);
                DOM_tbody.appendChild(DOM_tr);
                DOM_tr = undefined;
            } else {
                DOM_tr = document.createElement("tr");
                DOM_tr.appendChild(DOM_td);
            }
            if (skill.indexOf("Fluency(") === 0) {
                var dialect = skill.slice(8,-1);
                DOM_option = new Option(dialect,dialect,true,false);
                childhoodfluency.appendChild(DOM_option);
            }
        }
        // one cell row at the end?
        if (DOM_tr) {
            DOM_td = document.createElement("td");
            DOM_tr.appendChild(DOM_td);
            DOM_tbody.appendChild(DOM_tr);
        }
        
        DOM_table.appendChild(DOM_tbody);
        skilllist.appendChild(DOM_table);
    }
    
    // Other language
    DOM_h4 = document.createElement("h4");
    DOM_h4.appendChild(document.createTextNode("Add unlisted Language"));
    skilllist.appendChild(DOM_h4);
    DOM_form = document.createElement("form");
    DOM_table = document.createElement("table");
    DOM_table.style.width = "100%";
    DOM_table.style.tableLayout = "fixed";
    DOM_tbody = document.createElement("tbody");
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 3;
    DOM_td.appendChild(Character.createTextArea("skill-add-language",1,20));
    DOM_tr.appendChild(DOM_td);
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 1;
    DOM_button = Character.createButton("Add",Character.returnFalse(function(){
        var name = document.getElementById("skill-add-language").value;
        Character.addOtherSkill("Fluency("+name+")",["int"]);
        Character.addOtherSkill("Literacy("+name+")",["int"]);
    }));
    DOM_td.appendChild(DOM_button);
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    DOM_table.appendChild(DOM_tbody);
    DOM_form.appendChild(DOM_table);
    skilllist.appendChild(DOM_form);

    // Other Religion
    DOM_h4 = document.createElement("h4");
    DOM_h4.appendChild(document.createTextNode("Add unlisted Religion"));
    skilllist.appendChild(DOM_h4);
    DOM_form = document.createElement("form");
    DOM_table = document.createElement("table");
    DOM_table.style.width = "100%";
    DOM_table.style.tableLayout = "fixed";
    DOM_tbody = document.createElement("tbody");
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 3;
    DOM_td.appendChild(Character.createTextArea("skill-add-religion",1,20));
    DOM_tr.appendChild(DOM_td);
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 1;
    DOM_button = Character.createButton("Add",Character.returnFalse(function(){
        var name = document.getElementById("skill-add-religion").value;
        Character.addOtherSkill("Ritual("+name+")",["will"]);
        Character.addOtherSkill("Theology("+name+")",["int"]);
    }));
    DOM_td.appendChild(DOM_button);
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    DOM_table.appendChild(DOM_tbody);
    DOM_form.appendChild(DOM_table);
    skilllist.appendChild(DOM_form);

    // Other Ritual
    DOM_h4 = document.createElement("h4");
    DOM_h4.appendChild(document.createTextNode("Add unlisted Ritual"));
    skilllist.appendChild(DOM_h4);
    DOM_form = document.createElement("form");
    DOM_table = document.createElement("table");
    DOM_table.style.width = "100%";
    DOM_table.style.tableLayout = "fixed";
    DOM_tbody = document.createElement("tbody");
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 3;
    DOM_td.appendChild(Character.createTextArea("skill-add-ritual",1,20));
    DOM_tr.appendChild(DOM_td);
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 1;
    DOM_button = Character.createButton("Add",Character.returnFalse(function(){
        var name = document.getElementById("skill-add-ritual").value;
        Character.addOtherSkill("Ritual("+name+")",["will"]);
    }));
    DOM_td.appendChild(DOM_button);
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    DOM_table.appendChild(DOM_tbody);
    DOM_form.appendChild(DOM_table);
    skilllist.appendChild(DOM_form);

    // The 'other' skills
    DOM_h4 = document.createElement("h4");
    DOM_h4.appendChild(document.createTextNode("Add Unlisted Skill"));
    skilllist.appendChild(DOM_h4);
    DOM_form = document.createElement("form");
    DOM_table = document.createElement("table");
    DOM_table.style.width = "100%";
    DOM_table.style.tableLayout = "fixed";
    DOM_tbody = document.createElement("tbody");
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 4;
    DOM_td.appendChild(Character.createTextArea("skill-add-name",1,24));
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 4;
    DOM_td.appendChild(document.createTextNode("Attributes"));
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    var addStats = function(tbody,stats) {
        var DOM_tr = document.createElement("tr");
        for (var i=0; i<stats.length; i++) {
            var DOM_td = document.createElement("td");
            DOM_td.className = "right-align";
            DOM_td.appendChild(document.createTextNode(stats[i]+" "));
            var DOM_checkbox = document.createElement("input");
            DOM_checkbox.type = "checkbox";
            DOM_checkbox.id = "skill-add-"+stats[i];
            DOM_checkbox.checked = false;
            DOM_td.appendChild(DOM_checkbox);
            DOM_tr.appendChild(DOM_td);
        }
        DOM_tbody.appendChild(DOM_tr);
    };
    addStats(DOM_tbody,Character.allstats.slice(0,4));
    addStats(DOM_tbody,Character.allstats.slice(4));
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 4;
    DOM_button = Character.createButton("Add New Skill",Character.returnFalse(function(){
        var name = document.getElementById("skill-add-name").value;
        var stats = [];
        for(var i=0; i<Character.allstats.length; i++) {
            if (document.getElementById("skill-add-"+Character.allstats[i]).checked) {
                stats.push(Character.allstats[i]);
            }
        }
        Character.addOtherSkill(name,stats);
    }));
    DOM_td.appendChild(DOM_button);
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    DOM_table.appendChild(DOM_tbody);
    DOM_form.appendChild(DOM_table);
    skilllist.appendChild(DOM_form);
};

Character.setupButtons = function() {
    document.getElementById("birthdayminus1year").onclick = Character.birthdaydelta(-365*24*60*60*1000); 
    document.getElementById("birthdayminus1month").onclick = Character.birthdaydelta(-30*24*60*60*1000);
    document.getElementById("birthdayminus1day").onclick = Character.birthdaydelta(-24*60*60*1000);
    document.getElementById("birthdayplus1day").onclick = Character.birthdaydelta(24*60*60*1000);
    document.getElementById("birthdayplus1month").onclick = Character.birthdaydelta(30*24*60*60*1000);
    document.getElementById("birthdayplus1year").onclick = Character.birthdaydelta(365*24*60*60*1000);

    document.getElementById("gamedayminus1year").onclick = Character.gamedaydelta(-365*24*60*60*1000); 
    document.getElementById("gamedayminus1month").onclick = Character.gamedaydelta(-30*24*60*60*1000);
    document.getElementById("gamedayminus1day").onclick = Character.gamedaydelta(-24*60*60*1000);
    document.getElementById("gamedayplus1day").onclick = Character.gamedaydelta(24*60*60*1000);
    document.getElementById("gamedayplus1month").onclick = Character.gamedaydelta(30*24*60*60*1000);
    document.getElementById("gamedayplus1year").onclick = Character.gamedaydelta(365*24*60*60*1000);

    document.getElementById("gamedatesync").onchange = Character.returnFalse(function() {
        character.syncWithGlobalDate = document.getElementById("gamedatesync").checked;
        document.getElementById("gamedayminus1year").disabled = character.syncWithGlobalDate;
        document.getElementById("gamedayminus1month").disabled = character.syncWithGlobalDate;
        document.getElementById("gamedayminus1day").disabled = character.syncWithGlobalDate;
        document.getElementById("gamedayplus1day").disabled = character.syncWithGlobalDate;
        document.getElementById("gamedayplus1month").disabled = character.syncWithGlobalDate;
        document.getElementById("gamedayplus1year").disabled = character.syncWithGlobalDate;
        if (character.syncWithGlobalDate) {
            character.gametime = options.gametime;
            character.gametime = options.gametime;
            character.gameday = character.genre.dayMonthYearAsText(character.gametime,character.gamemonth);
            character.gamemonth = character.genre.asMonth(character.gametime);
            character.gamemonthtext = character.genre.dateMonthAsText(character.gamemonth);
            character.periods[character.periods.length-1].endmonth = character.gamemonth - character.birthmonth;
            character.periods[character.periods.length-1].endmonthtext = character.gamemonthtext;
            Character.incrementVersion();
            Character.populateAll();
        }
    });
    
    document.getElementById("genre").onchange = Character.returnFalse(function() {
        var sc = document.getElementById("genre");
        var oldgenre = character.genre;
        character.genre = new Genre(sc[sc.selectedIndex].text);
        character.birthmonth = character.genre.asMonth(character.birthtime);
        character.gamemonth = character.genre.asMonth(character.gametime);
        character.birthmonthtext = character.genre.dateMonthAsText(character.birthmonth);
        character.gamemonthtext = character.genre.dateMonthAsText(character.gamehmonth);
        if (!character.genre.equals(oldgenre)) {
            for(var i=0; i<character.periods.length; i++) {
                character.periods[i].childhoodLanguage = character.genre.getDefaultLanguage();
            }
        }
        
        // Reset equipment
        character.armour = Rules.armour[0];
        character.shield = Rules.shields[0];
        character.shieldLocations = [];
        character.favouredWeapons = [];
        
        Character.updateMonths();
        Character.setTextArea("djv_genre",character.genre.toString());
        Character.incrementVersion();
        Character.populateAll();
    });

    document.getElementById("nametextarea").onchange = Character.returnFalse(function() {
        character.name =  document.getElementById("nametextarea").value;
        Character.setTextArea("djv_nametextarea",character.name);
        Character.incrementVersion();
        Character.populateJSON();
    });

    document.getElementById("playernametextarea").onchange = Character.returnFalse(function() {
        character.playerName =  document.getElementById("playernametextarea").value;
        Character.setTextArea("djv_playernametextarea",character.playerName);
        Character.incrementVersion();
        Character.populateJSON();
    });

    document.getElementById("characterdescriptiontextarea").onchange = Character.returnFalse(function() {
        character.description =  document.getElementById("characterdescriptiontextarea").value;
        Character.incrementVersion();
        Character.populateJSON();
    });

    for(var i=0; i<Character.allstats.length; i++) {
        var stat = Character.allstats[i];
        document.getElementById(stat+"swapsel").onchange = Character.statSwap(stat,document.getElementById(stat+"swapsel"));
    }

    document.getElementById("reroll").onclick = Character.returnFalse(function() {
        var newcharacter = Rules.newCharacter();
        for(var i=0; i<Character.allstats.length; i++) {
            character[Character.allstats[i]] = newcharacter[Character.allstats[i]];
        }
        character.characterVersion = 0;
        character.lastEditDate = (new Date(Date.now())).toLocaleString();
        character.morale= Rules.startingMorale(character);
        character.woundPoints = Rules.woundPoints(character);
        character.interestPoints = Rules.interestPoints(character);
        Character.populateStats();
        Character.incrementVersion();
        Character.populateJSON();
    });

    document.getElementById("statsedit").onclick = Character.returnFalse(function(){
        var updateStat = function(stat,element) {
            return Character.returnFalse(function() {
                character[stat] = Character.validateStat(stat,element.value);
                Character.updateDerivedStats();
                Character.incrementVersion();
            });
        };
        for(var i=0; i<Character.allstats.length; i++) {
            var oldE = document.getElementById("stats-"+Character.allstats[i]);
            var newE = document.createElement("td");
            var textarea = document.createElement("textarea");
            textarea.value = character[Character.allstats[i]];
            textarea.rows = 1;
            textarea.cols = 8;
            textarea.className = "onerow";
            textarea.onchange = updateStat(Character.allstats[i],textarea);
            textarea.id = "stats-"+Character.allstats[i];
            newE.appendChild(textarea);
            oldE.parentNode.replaceChild(newE,oldE);
            Character.deleteElement(Character.allstats[i]+"swap");
        }
        document.getElementById("statsedit").disabled = true;
    });
    
    document.getElementById("statsedit").disabled = false; // Ensure enabled even after a reload of a page with it disabled
    
    document.getElementById("statslock").onclick = Character.returnFalse(function() {
        character.statslock = true;
        for(var i=0; i<Character.allstats.length; i++) {
            var oldE = document.getElementById("stats-"+Character.allstats[i]);
            var newE = document.createElement("td");
            var text = document.createTextNode(character[Character.allstats[i]].toString());
            newE.appendChild(text); 
            newE.id = "stats-"+Character.allstats[i];
            oldE.parentNode.replaceChild(newE,oldE);
        }
        Character.populateStats();
        Character.incrementVersion();
        Character.populateJSON();
    });
    
    document.getElementById("addPeriod").onclick = Character.returnFalse(function() {
        var op = character.periods[character.periods.length-1];
        var np = JSON.parse(JSON.stringify(op));
        // Split the time in two
        var duration = Math.round((op.endmonth - op.startmonth + 1)/2);
        op.endmonth = op.startmonth + duration - 1;
        np.startmonth = op.endmonth + 1;
        op.endmonthtext = character.genre.dateMonthAsText(op.endmonth+character.birthmonth);
        np.startmonthtext = character.genre.dateMonthAsText(np.startmonth+character.birthmonth);
        np.title = "Period "+(character.periods.length+1);
        np.description = "The next stage in life";
        character.periods.push(np);
        Character.incrementVersion();
        Character.populateAll();
    });
    
    document.getElementById("startPeriod").onclick = Character.returnFalse(function() {
        var op = character.periods[character.periods.length-1];
        var np = JSON.parse(JSON.stringify(op));
        // Split the time in two
        np.startmonth = op.endmonth + 1;
        np.endmonth = op.endmonth;
        np.endmonthtext = character.genre.dateMonthAsText(np.endmonth+character.birthmonth);
        np.startmonthtext = character.genre.dateMonthAsText(np.startmonth+character.birthmonth);
        np.title = "Period "+(character.periods.length+1);
        np.description = "The next stage in life";
        character.periods.push(np);
        Character.incrementVersion();
        Character.populateAll();
    });

    document.getElementById("JSONtriggerfile").onclick = Character.returnFalse(function() {
        document.getElementById("JSONfile").click();
    });

    document.getElementById("JSONfile").onchange = Character.returnFalse(function() {
        var file = document.getElementById("JSONfile").files[0];
        if (file) {
            var reader = new FileReader();
            reader.onload = function() {
                if (document.getElementById("statedit")) {
                    document.getElementById("statsedit").checked = false;
                    document.getElementById("statsedit").onchange();
                }
                Character.updateWithNewJSON(reader.result,true);
                // Copy across the form fields to the ones actually sent (in the hidden diff)
                Character.populateJSON();
                Character.setTextArea("djv_nametextarea",character.name);
                Character.setTextArea("djv_genre",character.genre.toString());
                Character.setTextArea("djv_playernametextarea",character.playerName);
            };
            reader.onerror = function() {
                alert("Error reading JSON, error code:"+reader.error);
            };
            reader.readAsText(file);
        }
    });
};

Character.initPage(); // Page first so that the new skills (if any) get loaded
Character.init();
Character.populateAll();
