//
// © Copyright 2014 David Vines
//
// Creative Commons License
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
//
// Big 'C' Character is used to namespace the functions
//
// little 'c' character contains the character being created.
Character.populateStats = function() {
  Character.setIdText("nametextarea",character.name);
  Character.setIdText("playernametextarea",character.playerName);
  Character.setIdText("genre",character.genre.toString());
  for(var i=0; i<Character.allstats.length; i++) {
      if (Character.edittableStats) {
          Character.setTextArea("stats-"+Character.allstats[i],character[Character.allstats[i]]);
      } else {
          Character.setIdText("stats-"+Character.allstats[i],character[Character.allstats[i]]);
      }
  }
  Character.setIdText("stats-morale",character.morale);
  Character.setIdText("stats-ip",character.interestPoints);

  Character.setIdText("characterdescriptiontextarea",character.description);

  Character.setIdText("stats-birthday",character.genre.dayMonthYearAsText(character.birthtime,character.birthmonth));
  Character.setIdText("stats-gameday",character.genre.dayMonthYearAsText(character.gametime,character.gamemonth));

  var lastperiodduration;
  if (character.periods.length === 1) {
      lastperiodduration = character.periods[character.periods.length-1].endmonth;
  } else {
      lastperiodduration = character.periods[character.periods.length-1].endmonth - character.periods[character.periods.length-2].endmonth;
  }
};

Character.updateNode = function(node,i) {
    if (node.nodeType === Node.ELEMENT_NODE) {
        node.id = node.id.replace("@@",i.toString());
        if (node.name !== undefined) {
            node.name = node.name.replace("@@",i.toString());
        }
        for(var j = 0; j < node.childNodes.length; j++) {
            Character.updateNode(node.childNodes[j],i);
        }
    }
};

Character.updateForm = function(node,i) {
    if (node.nodeType === Node.ELEMENT_NODE) {
        node.id = node.id.replace("-interest0-","-interest"+i+"-");
        if (node.name !== undefined) {
            node.name = node.name.replace("-interest0-","-interest"+i+"-");
        }
        for(var j = 0; j < node.childNodes.length; j++) {
            Character.updateForm(node.childNodes[j],i);
        }
    }
};

Character.updateSkillRow = function(p,interest) {
    Character.setIdText("period"+p+"-interest"+interest+"-skill",character.periods[p].interests[interest].skill);
    Character.setIdText("period"+p+"-interest"+interest+"-points",character.periods[p].interests[interest].ip);
    if (character.periods[p].interests[interest].teacher) {
        Character.setIdText("period"+p+"-interest"+interest+"-teacher","Teacher Bonus");
    }
};

Character.addSkillRow = function(p,interest) {
    Character.updateSkillRow(p,interest);
};

Character.addInterestKnowingPeriod = function(p,table,skillname) {
    var adding = true;
    for(var i=0; i<character.periods[p].interests.length; i++) {
        if (character.periods[p].interests[i].skill == skillname) {
            character.periods[p].iptotal += 1;
            character.periods[p].interests[i].ip += 1;
            Character.updateSkillRow(p,i);
            adding = false;
        }
    }
    if (adding) {
        var next = character.periods[p].interests.length;
        character.periods[p].interests.push(JSON.parse(JSON.stringify(Character.templates.interest)));
        character.periods[p].interests[next].skill = skillname;
        character.periods[p].interests[next].ip = 1;
        character.periods[p].iptotal += 1;
        Character.addInterestRow(p,table,next);
    }
    Character.populateSkills();
    Character.populateJSON();
};

Character.populatePeriods = function() {
    // Delete the old periods
    var i = 0;
    var period = document.getElementById("period"+i);
    while(period) {
        Character.deleteElement(period);
        i++;
        period = document.getElementById("period"+i);
    }

    // And build new ones
    var periods = document.getElementById("periods");
    var template = document.getElementById("period@@");

    for(i = 0; i < character.periods.length; i++) {
        var newNode = template.cloneNode(true);
        newNode.className = "tabbertab";
        newNode.title = character.periods[i].title;
        Character.updateNode(newNode,i);
        periods.parentNode.insertBefore(newNode,periods);
        
        // Header fields
        Character.setIdText("period"+i+"-descriptiontextarea",character.periods[i].description);
        
        // dates
        Character.setIdText("period"+i+"-startdate","Age: "+Character.durationLength(character.periods[i].startmonth),character.genre.dateMonthAsText(character.periods[i].startmonth+character.birthmonth));
        Character.setIdText("period"+i+"-enddate","Age: "+Character.durationLength(character.periods[i].endmonth),character.genre.dateMonthAsText(character.periods[i].endmonth+character.birthmonth));
        Character.setIdText("period"+i+"-enddatelength"," A period of "+Character.durationLength(character.periods[i].endmonth - character.periods[i].startmonth  + 1));
        
        Character.populatePeriodMonths(i);
        
        // childhood fluency
        if (character.periods[i].startmonth > 180) {
            Character.deleteElement(document.getElementById("period"+i+"-fluencyheader"));
            Character.deleteElement(document.getElementById("period"+i+"-fluencycontent"));
        } else {
            Character.setIdText("period"+i+"-childhoodfluencylanguage",character.periods[i].childhoodLanguage);
            Character.updateFluencyPoints(i);
        }
        
        // Interests
        var table = document.getElementById("period"+i+"-details").firstChild.nextSibling;
        for(var interest=0; interest < character.periods[i].interests.length; interest++) {
            Character.addInterestRow(i,table,interest);
        }
        Character.updateStatus(i);
    }
    
    Character.updateTabs();
};

Character.addOtherSkill = function(name,stats) {
    if (name !== "") {
        Rules.addSkill(name,stats);
        var newSkills = {};
        newSkills[name] = stats;
        character.newSkills.push(newSkills);
    }
    
    // And give the character the skill if they're on a period
    Character.addInterest(name)();
};

Character.populateJSON = function() {
};

Character.resetOtherSkillsList = function() {
};

Character.setupButtons = function() {
};

Character.initPage(); // Page first so that the new skills (if any) get loaded
Character.populateAll();
